// ======================================================================
//
// ControlMessageQueue.cpp
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "ControlMessageQueue.h"

// ======================================================================

ControlMessageQueue::ControlMessageQueue(int argQueueMax)
{
	queueMax = argQueueMax;
	queue = NEW ControlMessageQueueEntry[queueMax];

	writeLock=false;
	readLock=false;
	queueStart=0;
	queueIterator=0;
	queueEnd=0;
	queueMarker=-1;

	resetIterator();
}

// ----------------------------------------------------------------------

ControlMessageQueue::~ControlMessageQueue(void)
{
	delete [] queue;
	queue = NULL;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::resetToMarker(void)  // this also clears the marker
{
	DEBUG_FATAL(queueMarker==-1,("ControlMessageQueue::resetToMarker - queue was not marked before resetting.\n"));
	queueStart=queueMarker;
	queueMarker=-1;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::setMarker(void)
{
	queueMarker=queueEnd;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::event(ControlMessage message, real value, void *data)
{
	DEBUG_FATAL(writeLock, ("ControlMessageQueue::event - Destination Queue is writeLocked."));

	int newEnd;

	newEnd=queueEnd+1;
	if (newEnd==queueMax)
		newEnd=0;
	DEBUG_FATAL(newEnd==queueStart,("ControlMessageQueue::event - queue full.\n"));

	if (newEnd!=queueStart)  // ignore new events if the queue is full
	{
		queue[queueEnd].message=message;
		queue[queueEnd].value=value;
		queue[queueEnd].data=data;
		queueEnd=newEnd;
	}
}

// ----------------------------------------------------------------------

void ControlMessageQueue::copyToMarker(const ControlMessageQueue *copyMe)
{
	ControlMessageQueueEntry entry;

	DEBUG_FATAL(copyMe->readLock, ("ControlMessageQueue::copyToMarker - Source Queue is readLocked."));
	DEBUG_FATAL(writeLock, ("ControlMessageQueue::copyToMarker - Destination Queue is writeLocked."));

	queueMarker=-1;
	queueStart=0;
	queueEnd=0;

	copyMe->resetIterator();
	while (!copyMe->atEnd())
	{
		copyMe->getCurrentEntry(&entry.message,&entry.value,&entry.data);
		event(entry.message,entry.value,entry.data);
		copyMe->incrementIterator();
	}
}

// ----------------------------------------------------------------------

void ControlMessageQueue::appendToMarker(const ControlMessageQueue *copyMe)
{
	ControlMessageQueueEntry entry;

	DEBUG_FATAL(copyMe->readLock, ("ControlMessageQueue::appendToMarker - Source Queue is readLocked."));
	DEBUG_FATAL(writeLock, ("ControlMessageQueue::appendToMarker - Destination Queue is writeLocked."));

	copyMe->resetIterator();
	while (!copyMe->atEnd())
	{
		copyMe->getCurrentEntry(&entry.message,&entry.value,&entry.data);
		event(entry.message,entry.value,entry.data);
		copyMe->incrementIterator();
	}
}

// ----------------------------------------------------------------------

void ControlMessageQueue::resetIterator(void) const
{
	DEBUG_FATAL(readLock, ("ControlMessageQueue::resetIterator - Queue is readLocked."));
	queueIterator=queueStart;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::incrementIterator(void) const
{
	DEBUG_FATAL(readLock, ("ControlMessageQueue::incrementIterator - Queue is readLocked."));
	if (queueIterator!=queueMarker)
		if (++queueIterator==queueMax)  // wrap properly at the end of the queue
			queueIterator=0;
}

// ----------------------------------------------------------------------

bool ControlMessageQueue::atEnd(void) const  // you can NEVER iterate past the marker
{
	DEBUG_FATAL(readLock, ("ControlMessageQueue::atEnd - Queue is readLocked."));
	return queueIterator==queueMarker;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::getCurrentEntry(ControlMessage *message, real *value, void **data) const
{
	DEBUG_FATAL(readLock, ("ControlMessageQueue::getCurrentEntry - Queue is readLocked."));
	DEBUG_FATAL(queue[queueIterator].message==CM_nothing,("ControlMessageQueue::getCurrentEntry - attempted to get a deleted entry.\n"));

	if (message)
		*message = queue[queueIterator].message;

	if (value)
		*value = queue[queueIterator].value;

	if (data)
		*data = queue[queueIterator].data;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::eraseEntry(void)
{
	DEBUG_FATAL(writeLock, ("ControlMessageQueue::eraseEntry - Queue is writeLocked."));

	queue[queueIterator].message = CM_nothing;
	queue[queueIterator].value   = CONST_REAL(0);
	queue[queueIterator].data    = NULL;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::lockRead(void)
{
	readLock=true;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::unlockRead(void)
{
	readLock=false;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::lockWrite(void)
{
	writeLock=true;
}

// ----------------------------------------------------------------------

void ControlMessageQueue::unlockWrite(void)
{
	writeLock=false;
}

// ======================================================================
